/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.api.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import org.springblade.common.cache.ParamCache;
import org.springblade.common.constant.CacheBizConstant;
import org.springblade.common.constant.CommonConstant;
import org.springblade.common.enums.YesOrNoEnum;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.core.sms.SmsTemplate;
import org.springblade.core.sms.model.SmsCode;
import org.springblade.core.sms.model.SmsData;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.StringUtil;
import org.springblade.modules.activities.entity.Activities;
import org.springblade.modules.activities.service.IActivitiesService;
import org.springblade.modules.activities.vo.ActivitiesVO;
import org.springblade.modules.activities.wrapper.ActivitiesWrapper;
import org.springblade.modules.mall.entity.GoodsInfo;
import org.springblade.modules.mall.entity.GoodsSpec;
import org.springblade.modules.mall.service.IGoodsInfoService;
import org.springblade.modules.mall.service.IGoodsSpecService;
import org.springblade.modules.mall.vo.GoodsInfoVO;
import org.springblade.modules.mall.wrapper.GoodsInfoWrapper;
import org.springblade.modules.platform.entity.Ads;
import org.springblade.modules.platform.entity.Payments;
import org.springblade.modules.platform.service.IAdsService;
import org.springblade.modules.platform.service.IPaymentsService;
import org.springblade.modules.resource.builder.sms.SmsBuilder;
import org.springblade.modules.resource.utils.SmsUtil;
import org.springblade.modules.system.entity.Area;
import org.springblade.modules.system.entity.DictBiz;
import org.springblade.modules.system.service.IAreaService;
import org.springblade.modules.system.service.IDictBizService;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import static org.springblade.modules.resource.utils.SmsUtil.*;

/**
 * 控制器
 *
 * @author xuwei
 * @since 2022-02-12
 */
@RestController
@AllArgsConstructor
@RequestMapping(CommonConstant.API_URL)
@Api(value = "公共接口", tags = "02.公共接口")
public class CommonController extends BladeController {

	private final IPaymentsService paymentsService;
	private final IAdsService adsService;
	private final SmsBuilder smsBuilder;
	private final BladeRedis bladeRedis;
	private final IAreaService areaService;
	private final IActivitiesService activitiesService;
	private final IDictBizService dictService;
	private final IGoodsInfoService goodsInfoService;
	private final IGoodsSpecService goodsSpecService;

	@GetMapping("/payments-list")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "查询支付方式")
	public R<List<Payments>> paymentsList() {
		List<Payments> list =  bladeRedis.get(CacheBizConstant.CACHE_DEFAULT_PAYMENTS);
		if (list == null || list.size() == 0){
			Payments payments = new Payments();
			payments.setIsEnabled(2);
			payments.setPayFor(4);//APP端
			list = paymentsService.list(Condition.getQueryWrapper(payments).lambda().orderByAsc(Payments::getSort));
			bladeRedis.setEx(CacheBizConstant.CACHE_DEFAULT_PAYMENTS,list, CacheBizConstant.CACHE_TIME);
		}
		return R.data(list);
	}

	@GetMapping("/ads-list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "查询轮播数据", notes = "传入类型和条数,类型为字典项ads_type，1应用首页，2商城首页，3推荐品牌，4抢购专区，5新品推荐，6人气推荐")
	public R<List<Ads>> adsList(@ApiParam(value = "类型", required = true)  @RequestParam Integer type,
								@ApiParam(value = "条数，默认条数由系统配置", required = false)  @RequestParam(required = false) Integer limit) {
		if(ObjectUtil.isNull(limit)){
			limit = Integer.valueOf(ParamCache.getValue(CommonConstant.DEFAULT_ADS_LIMIT));
		}
		return R.data(adsService.selectAdsList(type, limit));
	}

	@SneakyThrows
	@PostMapping("/common/send-validate")
	@ApiOperation(value = "短信发送", notes = "传入手机号")
	@ApiOperationSupport(order = 3)
	public R sendValidate(@RequestParam String phone) {
		Map<String, String> params = SmsUtil.getValidateParams();
		SmsCode smsCode = smsBuilder.noAuthTemplate().sendValidate(new SmsData(params).setKey(PARAM_KEY), phone);
		return smsCode.isSuccess() ? R.data(smsCode, SEND_SUCCESS) : R.fail("发送频率过高，请稍后再试");
	}

	@ApiOperation(value = "行政区域列表查询", notes = "行政区域列表查询")
	@ApiOperationSupport(order = 4)
	@GetMapping("/common/area")
	public List<Area> listDistinct(@ApiParam(value = "行政区域父id，省的pid为0", required = true) @RequestParam("pid") String pid) {
		return areaService.lambdaQuery().eq(Area::getPid, pid).list();
	}

	@GetMapping("/activities/list")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "活动列表", notes = "传入activities")
	public R<List<ActivitiesVO>> list(Query query) {
		IPage<Activities> pages = activitiesService.page(Condition.getPage(query));
		List<Activities> records = pages.getRecords();
		return R.data(ActivitiesWrapper.build().listVO(records));
	}

	/**
	 * 获取字典
	 */
	@GetMapping("/dictionary")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "获取字典", notes = "获取字典 wine_year 年份，wine_taste 口感")
	public R<List<DictBiz>> dictionary(String code) {
		List<DictBiz> tree = dictService.getList(code);
		return R.data(tree);
	}


	@GetMapping("/goods-by-type")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "根据类型获取单个商品", notes = "1-免费领酒")
	public R<GoodsInfoVO> goodsByType(String type) {
		if (StrUtil.isBlank(type)) {
			return R.fail("参数[type]不能为空");
		}
		if ("1".equals(type)) {
			String value = ParamCache.getValueDefault(CommonConstant.FREE_WINE_GOODS_ID, "");
			if (StrUtil.isBlank(value)) {
				return R.fail("请联系管理员设置参数");
			}
			GoodsInfo goodsInfo = goodsInfoService.getById(value);
			Assert.notNull(goodsInfo, "商品信息不存在");
			// 获取默认规格数据
			List<GoodsSpec> list = goodsSpecService.lambdaQuery().eq(GoodsSpec::getGoodsId, goodsInfo.getId()).eq(GoodsSpec::getIsDefault, YesOrNoEnum.YES.getValue()).list();
			Assert.notEmpty(list, "商品规格数据不存在");
			// 校验默认规格是否只有一条
			if (list.size() > 1) {
				return R.fail("商品规格数据异常");
			}
			GoodsInfoVO goodsInfoVO = GoodsInfoWrapper.build().entityVO(goodsInfo);
			goodsInfoVO.setGoodsSpecList(list);
			// 快递包装费，后台设置
			goodsInfoVO.setWlPrice(Double.parseDouble(ParamCache.getValueDefault(CommonConstant.FREE_WINE_GOODS_WL_PRICE, "28")));
			return data(goodsInfoVO);
		} else {
			return R.fail("该数据类型不存在");
		}
	}

}
