/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.custom.controller;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.custom.entity.BaseWineLabel;
import org.springblade.modules.custom.entity.BottleCapLabel;
import org.springblade.modules.custom.service.IBaseWineLabelService;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.custom.entity.BaseWine;
import org.springblade.modules.custom.vo.BaseWineVO;
import org.springblade.modules.custom.service.IBaseWineService;
import org.springblade.core.boot.ctrl.BladeController;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 基酒表 控制器
 *
 * @author BladeX
 * @since 2023-02-08
 */
@RestController
@AllArgsConstructor
@RequestMapping("custom/basewine")
@Api(value = "基酒表", tags = "基酒表接口")
public class BaseWineController extends BladeController {

	private final IBaseWineService baseWineService;
	private final IBaseWineLabelService baseWineLabelService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入baseWine")
	public R<BaseWine> detail(BaseWine baseWine) {
		BaseWine detail = baseWineService.getOne(Condition.getQueryWrapper(baseWine));
		List<BaseWineLabel> list = baseWineLabelService.lambdaQuery().eq(BaseWineLabel::getBaseWineId, detail.getId()).list();
		if (CollUtil.isNotEmpty(list)) {
			detail.setLabelIds(CollUtil.join(list.stream().map(BaseWineLabel::getLabelId).collect(Collectors.toList()), ","));
		}
		return R.data(detail);
	}

	/**
	 * 分页 基酒表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入baseWine")
	public R<IPage<BaseWine>> list(BaseWine baseWine, Query query) {
		QueryWrapper<BaseWine> queryWrapper = Condition.getQueryWrapper(baseWine);
		if (!AuthUtil.getUserRole().contains("admin")) {
			queryWrapper.eq("create_user_id", AuthUtil.getUserId());
		}
		IPage<BaseWine> pages = baseWineService.page(Condition.getPage(query), queryWrapper);
		return R.data(pages);
	}

	/**
	 * 自定义分页 基酒表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入baseWine")
	public R<IPage<BaseWineVO>> page(BaseWineVO baseWine, Query query) {
		IPage<BaseWineVO> pages = baseWineService.selectBaseWinePage(Condition.getPage(query), baseWine);
		return R.data(pages);
	}

	/**
	 * 新增 基酒表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入baseWine")
	public R save(@Valid @RequestBody BaseWine baseWine) {
		return R.status(baseWineService.save(baseWine));
	}

	/**
	 * 修改 基酒表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入baseWine")
	public R update(@Valid @RequestBody BaseWine baseWine) {
		return R.status(baseWineService.updateById(baseWine));
	}

	/**
	 * 新增或修改 基酒表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入baseWine")
	public R submit(@Valid @RequestBody BaseWine baseWine) {
		boolean b = baseWineService.saveOrUpdate(baseWine);
		List<BaseWineLabel> list = baseWineLabelService.lambdaQuery().eq(BaseWineLabel::getBaseWineId, baseWine.getId()).list();
		if (CollUtil.isNotEmpty(list)) {
			baseWineLabelService.removeByIds(list.stream().map(BaseWineLabel::getId).collect(Collectors.toList()));
		}
		String[] split = baseWine.getLabelIds().split(",");
		List<BaseWineLabel> bottleCapLabelList = new ArrayList<>();
		for (String s : split) {
			BaseWineLabel bottleCapLabel = new BaseWineLabel();
			bottleCapLabel.setBaseWineId(baseWine.getId());
			bottleCapLabel.setLabelId(Long.parseLong(s));
			bottleCapLabelList.add(bottleCapLabel);
		}
		baseWineLabelService.saveBatch(bottleCapLabelList);
		return R.status(b);
	}

	
	/**
	 * 删除 基酒表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(baseWineService.removeByIds(Func.toLongList(ids)));
	}

	
}
