/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.custom.controller;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.custom.entity.BottleBodyLabel;
import org.springblade.modules.custom.entity.Manufacturer;
import org.springblade.modules.custom.service.IBottleBodyLabelService;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.custom.entity.BottleBody;
import org.springblade.modules.custom.vo.BottleBodyVO;
import org.springblade.modules.custom.wrapper.BottleBodyWrapper;
import org.springblade.modules.custom.service.IBottleBodyService;
import org.springblade.core.boot.ctrl.BladeController;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 瓶身表 控制器
 *
 * @author BladeX
 * @since 2023-02-07
 */
@RestController
@AllArgsConstructor
@RequestMapping("custom/bottlebody")
@Api(value = "瓶身表", tags = "瓶身表接口")
public class BottleBodyController extends BladeController {

	private final IBottleBodyService bottleBodyService;
	private final IBottleBodyLabelService bottleBodyLabelService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入bottleBody")
	public R<BottleBodyVO> detail(BottleBody bottleBody) {
		BottleBody detail = bottleBodyService.getOne(Condition.getQueryWrapper(bottleBody));
		List<BottleBodyLabel> list = bottleBodyLabelService.lambdaQuery().eq(BottleBodyLabel::getBottleBodyId, detail.getId()).list();
		if (CollUtil.isNotEmpty(list)) {
			detail.setLabelIds(CollUtil.join(list.stream().map(BottleBodyLabel::getLabelId).collect(Collectors.toList()), ","));
		}
		return R.data(BottleBodyWrapper.build().entityVO(detail));
	}

	/**
	 * 分页 瓶身表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入bottleBody")
	public R<IPage<BottleBodyVO>> list(BottleBody bottleBody, Query query) {
		QueryWrapper<BottleBody> queryWrapper = Condition.getQueryWrapper(bottleBody);
		if (!AuthUtil.getUserRole().contains("admin")) {
			queryWrapper.eq("create_user_id", AuthUtil.getUserId());
		}
		IPage<BottleBody> pages = bottleBodyService.page(Condition.getPage(query), queryWrapper);
		return R.data(BottleBodyWrapper.build().pageVO(pages));
	}


	/**
	 * 自定义分页 瓶身表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入bottleBody")
	public R<IPage<BottleBodyVO>> page(BottleBodyVO bottleBody, Query query) {
		IPage<BottleBodyVO> pages = bottleBodyService.selectBottleBodyPage(Condition.getPage(query), bottleBody);
		return R.data(pages);
	}

	/**
	 * 新增 瓶身表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入bottleBody")
	public R save(@Valid @RequestBody BottleBody bottleBody) {
		return R.status(bottleBodyService.save(bottleBody));
	}

	/**
	 * 修改 瓶身表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入bottleBody")
	public R update(@Valid @RequestBody BottleBody bottleBody) {
		return R.status(bottleBodyService.updateById(bottleBody));
	}

	/**
	 * 新增或修改 瓶身表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入bottleBody")
	public R submit(@Valid @RequestBody BottleBody bottleBody) {
		boolean b = bottleBodyService.saveOrUpdate(bottleBody);

		List<BottleBodyLabel> list = bottleBodyLabelService.lambdaQuery().eq(BottleBodyLabel::getBottleBodyId, bottleBody.getId()).list();
		if (CollUtil.isNotEmpty(list)) {
			bottleBodyLabelService.removeByIds(list.stream().map(BottleBodyLabel::getId).collect(Collectors.toList()));
		}
		String[] split = bottleBody.getLabelIds().split(",");
		List<BottleBodyLabel> bottleBodyLabelList = new ArrayList<>();
		for (String s : split) {
			BottleBodyLabel bottleBodyLabel = new BottleBodyLabel();
			bottleBodyLabel.setBottleBodyId(bottleBody.getId());
			bottleBodyLabel.setLabelId(Long.parseLong(s));
			bottleBodyLabelList.add(bottleBodyLabel);
		}
		bottleBodyLabelService.saveBatch(bottleBodyLabelList);
		return R.status(b);
	}

	
	/**
	 * 删除 瓶身表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(bottleBodyService.removeByIds(Func.toLongList(ids)));
	}

	
}
