/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.custom.controller;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.custom.entity.BottleCap;
import org.springblade.modules.custom.entity.BottleLabelUnion;
import org.springblade.modules.custom.entity.BottleLabelUnion;
import org.springblade.modules.custom.service.IBottleLabelUnionService;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.custom.entity.BottleLabel;
import org.springblade.modules.custom.vo.BottleLabelVO;
import org.springblade.modules.custom.wrapper.BottleLabelWrapper;
import org.springblade.modules.custom.service.IBottleLabelService;
import org.springblade.core.boot.ctrl.BladeController;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 标贴表 控制器
 *
 * @author BladeX
 * @since 2023-02-07
 */
@RestController
@AllArgsConstructor
@RequestMapping("custom/bottlelabel")
@Api(value = "标贴表", tags = "标贴表接口")
public class BottleLabelController extends BladeController {

	private final IBottleLabelService bottleLabelService;
	private final IBottleLabelUnionService bottleLabelUnionService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入bottleLabel")
	public R<BottleLabelVO> detail(BottleLabel bottleLabel) {
		BottleLabel detail = bottleLabelService.getOne(Condition.getQueryWrapper(bottleLabel));
		BottleLabelVO bottleLabelVO = BottleLabelWrapper.build().entityVO(detail);
		List<BottleLabelUnion> list = bottleLabelUnionService.lambdaQuery().eq(BottleLabelUnion::getBottleLabelId, detail.getId()).list();
		if (CollUtil.isNotEmpty(list)) {
			bottleLabelVO.setLabelIds(CollUtil.join(list.stream().map(BottleLabelUnion::getLabelId).collect(Collectors.toList()), ","));
		}
		return R.data(bottleLabelVO);
	}

	/**
	 * 分页 标贴表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入bottleLabel")
	public R<IPage<BottleLabelVO>> list(BottleLabel bottleLabel, Query query) {
		QueryWrapper<BottleLabel> queryWrapper = Condition.getQueryWrapper(bottleLabel);
		if (!AuthUtil.getUserRole().contains("admin")) {
			queryWrapper.eq("create_user_id", AuthUtil.getUserId());
		}
		IPage<BottleLabel> pages = bottleLabelService.page(Condition.getPage(query), Condition.getQueryWrapper(bottleLabel));
		return R.data(BottleLabelWrapper.build().pageVO(pages));
	}


	/**
	 * 自定义分页 标贴表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入bottleLabel")
	public R<IPage<BottleLabelVO>> page(BottleLabelVO bottleLabel, Query query) {
		IPage<BottleLabelVO> pages = bottleLabelService.selectBottleLabelPage(Condition.getPage(query), bottleLabel);
		return R.data(pages);
	}

	/**
	 * 新增 标贴表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入bottleLabel")
	public R save(@Valid @RequestBody BottleLabel bottleLabel) {
		return R.status(bottleLabelService.save(bottleLabel));
	}

	/**
	 * 修改 标贴表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入bottleLabel")
	public R update(@Valid @RequestBody BottleLabel bottleLabel) {
		return R.status(bottleLabelService.updateById(bottleLabel));
	}

	/**
	 * 新增或修改 标贴表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入bottleLabel")
	public R submit(@Valid @RequestBody BottleLabel bottleLabel) {
		boolean b = bottleLabelService.saveOrUpdate(bottleLabel);
		List<BottleLabelUnion> list = bottleLabelUnionService.lambdaQuery().eq(BottleLabelUnion::getBottleLabelId, bottleLabel.getId()).list();
		if (CollUtil.isNotEmpty(list)) {
			bottleLabelUnionService.removeByIds(list.stream().map(BottleLabelUnion::getId).collect(Collectors.toList()));
		}
		String[] split = bottleLabel.getLabelIds().split(",");
		List<BottleLabelUnion> bottleBodyUnionList = new ArrayList<>();
		for (String s : split) {
			BottleLabelUnion bottleBodyUnion = new BottleLabelUnion();
			bottleBodyUnion.setBottleLabelId(bottleLabel.getId());
			bottleBodyUnion.setLabelId(Long.parseLong(s));
			bottleBodyUnionList.add(bottleBodyUnion);
		}
		bottleLabelUnionService.saveBatch(bottleBodyUnionList);
		return R.status(b);
	}

	
	/**
	 * 删除 标贴表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(bottleLabelService.removeByIds(Func.toLongList(ids)));
	}

	
}
