/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.custom.controller;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.common.enums.StatusEnum;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.modules.custom.entity.*;
import org.springblade.modules.custom.service.*;
import org.springblade.modules.custom.vo.BartenderVO;
import org.springblade.modules.custom.wrapper.BartenderWrapper;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.custom.vo.ManufacturerVO;
import org.springblade.modules.custom.wrapper.ManufacturerWrapper;
import org.springblade.core.boot.ctrl.BladeController;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 酒厂表 控制器
 *
 * @author BladeX
 * @since 2023-03-01
 */
@RestController
@AllArgsConstructor
@RequestMapping("custom/manufacturer")
@Api(value = "酒厂表", tags = "酒厂接口")
public class ManufacturerController extends BladeController {

	private final IManufacturerService manufacturerService;
	private final IManufacturerLabelService manufacturerLabelService;
	private final IManufacturerCertService manufacturerCertService;
	private final IBaseWineService baseWineService;
	private final IBartenderService bartenderService;
	private final IBartenderSkillService bartenderSkillService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入manufacturer")
	public R<ManufacturerVO> detail(Manufacturer manufacturer) {
		Manufacturer detail = manufacturerService.getOne(Condition.getQueryWrapper(manufacturer));
		ManufacturerVO manufacturerVO = ManufacturerWrapper.build().entityVO(detail);
		// 标签
		List<ManufacturerLabel> list = manufacturerLabelService.lambdaQuery().eq(ManufacturerLabel::getManufacturerId, detail.getId()).list();
		if (CollUtil.isNotEmpty(list)) {
			manufacturerVO.setLabelIds(CollUtil.join(list.stream().map(ManufacturerLabel::getLabelId).collect(Collectors.toList()), ","));
		}
		// 资质
		List<ManufacturerCert> certList = manufacturerCertService.lambdaQuery().eq(ManufacturerCert::getManufacturerId, detail.getId()).list();
		if (CollUtil.isNotEmpty(certList)) {
			manufacturerVO.setCertList(certList.stream().map(ManufacturerCert::getCertImage).collect(Collectors.toList()));
		}
		return R.data(manufacturerVO);
	}

	/**
	 * 分页 厂商表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入manufacturer")
	public R<IPage<ManufacturerVO>> list(Manufacturer manufacturer, Query query) {
		QueryWrapper<Manufacturer> queryWrapper = Condition.getQueryWrapper(manufacturer);
		if (!AuthUtil.getUserRole().contains("admin")) {
			queryWrapper.eq("create_user_id", AuthUtil.getUserId());
		}
		IPage<Manufacturer> pages = manufacturerService.page(Condition.getPage(query), queryWrapper);
		return R.data(ManufacturerWrapper.build().pageVO(pages));
	}


	/**
	 * 自定义分页 厂商表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入manufacturer")
	public R<IPage<ManufacturerVO>> page(ManufacturerVO manufacturer, Query query) {
		IPage<ManufacturerVO> pages = manufacturerService.selectManufacturerPage(Condition.getPage(query), manufacturer);
		return R.data(pages);
	}

	/**
	 * 新增 厂商表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入manufacturer")
	public R save(@Valid @RequestBody Manufacturer manufacturer) {
		return R.status(manufacturerService.save(manufacturer));
	}

	/**
	 * 修改 厂商表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入manufacturer")
	public R update(@Valid @RequestBody Manufacturer manufacturer) {
		return R.status(manufacturerService.updateById(manufacturer));
	}

	/**
	 * 新增或修改 厂商表
	 */
	@Transactional(rollbackFor = Exception.class)
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入manufacturer")
	public R submit(@Valid @RequestBody Manufacturer manufacturer) {
		boolean b = manufacturerService.saveOrUpdate(manufacturer);
		// 保存酒厂标签
		manufacturerLabelService.saveManufacturerLabel(manufacturer);
		// 保存酒厂资质
		manufacturerCertService.saveManufacturerCert(manufacturer);
		return R.status(b);
	}

	
	/**
	 * 删除 厂商表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(manufacturerService.removeByIds(Func.toLongList(ids)));
	}

	@GetMapping("/get")
	@ApiOperationSupport(order = 8)
	public R<List<Manufacturer>> get(Query query) {
		IPage<Manufacturer> page = manufacturerService.page(Condition.getPage(query));
		return data(page.getRecords());
	}

}
