/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.mall.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.common.enums.YesOrNoEnum;
import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.mall.entity.SeckillRound;
import org.springblade.modules.mall.vo.SeckillRoundVO;
import org.springblade.modules.mall.wrapper.SeckillRoundWrapper;
import org.springblade.modules.mall.service.ISeckillRoundService;
import org.springblade.core.boot.ctrl.BladeController;

import java.util.List;

/**
 * 限时秒杀场次表 控制器
 *
 * @author BladeX
 * @since 2023-05-04
 */
@RestController
@AllArgsConstructor
@RequestMapping("mall/seckillround")
@Api(value = "限时秒杀场次表", tags = "限时秒杀场次表接口")
public class SeckillRoundController extends BladeController {

	private final ISeckillRoundService seckillRoundService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入seckillRound")
	public R<SeckillRoundVO> detail(SeckillRound seckillRound) {
		SeckillRound detail = seckillRoundService.getOne(Condition.getQueryWrapper(seckillRound));
		return R.data(SeckillRoundWrapper.build().entityVO(detail));
	}


	@GetMapping("round")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "获取秒杀场次")
	public R<List<SeckillRoundVO>> round() {
		List<SeckillRound> list = seckillRoundService.lambdaQuery().eq(SeckillRound::getEnable, YesOrNoEnum.YES.getValue())
				.eq(SeckillRound::getIsDelete, YesOrNoEnum.NO.getValue())
				.orderByAsc(SeckillRound::getStartTime).list();
		return data(SeckillRoundWrapper.build().listVO(list));
	}

	/**
	 * 分页 限时秒杀场次表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入seckillRound")
	public R<IPage<SeckillRoundVO>> list(SeckillRound seckillRound, Query query) {
		IPage<SeckillRound> pages = seckillRoundService.page(Condition.getPage(query), Condition.getQueryWrapper(seckillRound));
		return R.data(SeckillRoundWrapper.build().pageVO(pages));
	}


	/**
	 * 自定义分页 限时秒杀场次表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入seckillRound")
	public R<IPage<SeckillRoundVO>> page(SeckillRoundVO seckillRound, Query query) {
		IPage<SeckillRoundVO> pages = seckillRoundService.selectSeckillRoundPage(Condition.getPage(query), seckillRound);
		return R.data(pages);
	}

	/**
	 * 新增 限时秒杀场次表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入seckillRound")
	public R save(@Valid @RequestBody SeckillRound seckillRound) {
		if (seckillRound.getStartTime().isAfter(seckillRound.getEndTime())) {
			return fail("开始时间不能小于结束时间");
		}
		return R.status(seckillRoundService.save(seckillRound));
	}

	/**
	 * 修改 限时秒杀场次表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入seckillRound")
	public R update(@Valid @RequestBody SeckillRound seckillRound) {
		if (seckillRound.getStartTime().isAfter(seckillRound.getEndTime())) {
			return fail("开始时间不能小于结束时间");
		}
		return R.status(seckillRoundService.updateById(seckillRound));
	}

	/**
	 * 新增或修改 限时秒杀场次表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入seckillRound")
	public R submit(@Valid @RequestBody SeckillRound seckillRound) {
		if (seckillRound.getStartTime().isAfter(seckillRound.getEndTime())) {
			return fail("开始时间不能小于结束时间");
		}
		return R.status(seckillRoundService.saveOrUpdate(seckillRound));
	}

	
	/**
	 * 删除 限时秒杀场次表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(seckillRoundService.removeByIds(Func.toLongList(ids)));
	}

	
}
